//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (Params.Storage != StorageType.Blank)  return string.Empty; #>
using System;
//!<# if (Params.ACL || Params.Hosting == HostingEnvironment.Listener) { #>
using System.Net;
//!<# } #>
//!<# if (Params.ACL || Params.Hosting == HostingEnvironment.IIS) { #>
using System.Web;
//!<# } #>
using ITHit.WebDAV.Server;

namespace ITHit.WebDav.Server.SampleModel.__DavImpl__
{
    /// <summary>
    /// Implementation of <see cref="DavContextBase"/>,
    /// it contains implementation for path-to-item resolution.
    /// </summary>
    public class DavContext : DavContextBase
    {
        //!<# if (Params.Hosting == HostingEnvironment.IIS) { #>
        /// <summary>
        /// Initializes a new instance of the DavContext class for IIS based solution.
        /// </summary>
        /// <param name="httpContext">Instance of <see cref="HttpContext"/>.</param>
        /// <param name="logger">Instance of <see cref="ILogger"/>.</param>
        public DavContext(HttpContext httpContext, ILogger logger) : base(httpContext)
        {
            throw new NotImplementedException();
        }
        //!<# } #>

        //!<# if (Params.Hosting == HostingEnvironment.Listener) { #>
        /// <summary>
        /// Initializes a new instance of the DavContext class for listener based solution.
        /// </summary>
        /// <param name="context">Instance of <see cref="HttpListenerContext"/>.</param>
        /// <param name="prefixes">Collection of listener prefixes.</param>
        /// <param name="logger">Instance of <see cref="ILogger"/>.</param>
        public DavContext(HttpListenerContext context, HttpListenerPrefixCollection prefixes, ILogger logger) :
            base(context, prefixes)
        {
            throw new NotImplementedException();
        }
        //!<# } #>

        /// <summary>
        /// Implementation of this abstract method is used by WebDAV engine to find hierarchy item objects by path.
        /// </summary>
        /// <param name="path">
        /// Path of the hierarchy item object.
        /// It is always the full path from the root of the WebDAV repository.
        /// </param>
        /// <returns>
        /// Hierarchy item object referenced by the specified path or <c>null</c>
        /// if hierarchy item not found.
        /// </returns>
        public override IHierarchyItem GetHierarchyItem(string path)
        {
            //!<# if (Params.ACL) { #>
            path = path.Trim();
            int ind = path.IndexOf('?');
            if (ind > 0)
            {
                path = path.Remove(ind);
            }

            path = path.TrimStart('/').TrimEnd('/');

            if (path == "acl")
            {
                return new AclFolder(this);
            }

            if (path == UserFolder.PREFIX)
            {
                return new UserFolder(this);
            }

            if (path == GroupFolder.PREFIX)
            {
                return new GroupFolder(this);
            }

            if (path.StartsWith(UserFolder.PATH))
            {
                string name = EncodeUtil.DecodeUrlPart(path.Substring(UserFolder.PATH.Length));
                throw new NotImplementedException();
            }

            if (path.StartsWith(GroupFolder.PATH))
            {
                string name = EncodeUtil.DecodeUrlPart(path.Substring(GroupFolder.PATH.Length));
                throw new NotImplementedException();
            }
            //!<# } #>

            //!<# if (false) {#>
            //TODO: create some fixed repository here so the sample is runnable and shows
            //some structure.
            //!<# } #>

            return null;
        }
    }
}
